package at.ac.uibk.gitsearch.es.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;

import java.util.List;
import java.util.Objects;

@JsonIgnoreProperties(ignoreUnknown = true)
public class ArtemisExerciseInfo extends ExerciseInfo {

    private Float maxPoints;

    private Float bonusPoints;

    private ExerciseMode mode;

    private Boolean staticCodeAnalysis;

    private Boolean allowOfflineIDE;

    private Boolean allowOnlineEditor;

    private Boolean showTestNamesToStudents;

    private Boolean sequentialTestRuns;

    public ArtemisExerciseInfo() {

    }

    public Float getMaxPoints() {
        return maxPoints;
    }

    public void setMaxPoints(Float maxPoints) {
        this.maxPoints = maxPoints;
    }

    public Float getBonusPoints() {
        return bonusPoints;
    }

    public void setBonusPoints(Float bonusPoints) {
        this.bonusPoints = bonusPoints;
    }

    public ExerciseMode getMode() {
        return mode;
    }

    public void setMode(ExerciseMode mode) {
        this.mode = mode;
    }

    public Boolean getStaticCodeAnalysis() {
        return staticCodeAnalysis;
    }

    public void setStaticCodeAnalysis(Boolean staticCodeAnalysis) {
        this.staticCodeAnalysis = staticCodeAnalysis;
    }

    public Boolean getAllowOfflineIDE() {
        return allowOfflineIDE;
    }

    public void setAllowOfflineIDE(Boolean allowOfflineIDE) {
        this.allowOfflineIDE = allowOfflineIDE;
    }

    public Boolean getAllowOnlineEditor() {
        return allowOnlineEditor;
    }

    public void setAllowOnlineEditor(Boolean allowOnlineEditor) {
        this.allowOnlineEditor = allowOnlineEditor;
    }

    public Boolean getShowTestNamesToStudents() {
        return showTestNamesToStudents;
    }

    public void setShowTestNamesToStudents(Boolean showTestNamesToStudents) {
        this.showTestNamesToStudents = showTestNamesToStudents;
    }

    public Boolean getSequentialTestRuns() {
        return sequentialTestRuns;
    }

    public void setSequentialTestRuns(Boolean sequentialTestRuns) {
        this.sequentialTestRuns = sequentialTestRuns;
    }

    public ExerciseInfo getExerciseInfoOnly() {
        ExerciseInfo exerciseInfo = new ExerciseInfo();
        exerciseInfo.setMetadataVersion(this.getMetadataVersion());
        exerciseInfo.setType(this.getType());
        exerciseInfo.setTitle(this.getTitle());
        exerciseInfo.setLicense(this.getLicense());
        if (this.getDescription() != null) {
            exerciseInfo.setDescription(this.getDescription());
        }
        if (this.getKeyword() != null) {
            exerciseInfo.setKeyword(this.getKeyword());
        }
        if (this.getFormat() != null) {
            exerciseInfo.setFormat(this.getFormat());
        }
        if (this.getStructure() != null) {
            exerciseInfo.setStructure(this.getStructure());
        }
        if (this.getProgrammingLanguage() != null) {
            exerciseInfo.setProgrammingLanguage(this.getProgrammingLanguage());
        }
        if (this.getLanguage() != null) {
            exerciseInfo.setLanguage(this.getLanguage());
        }
        if (this.getDifficulty() != null) {
            exerciseInfo.setDifficulty(this.getDifficulty());
        }
        if (this.getCreator() != null) {
            exerciseInfo.setCreator(List.copyOf(this.getCreator()));
        }
        if (this.getContributor() != null) {
            exerciseInfo.setContributor(List.copyOf(this.getContributor()));
        }
        if (this.getSource() != null) {
            exerciseInfo.setSource(this.getSource());
        }
        return exerciseInfo;
    }

    @JsonIgnore
    public ExerciseInfo getArtemisInfoOnly() {
        ArtemisExerciseInfo exerciseInfo = new ArtemisExerciseInfo();

        if (this.getMaxPoints() != null) {
            exerciseInfo.setMaxPoints(this.getMaxPoints());
        }

        if (this.getBonusPoints() != null) {
            exerciseInfo.setBonusPoints(this.getBonusPoints());
        }

        if (this.getMode() != null) {
            exerciseInfo.setMode(this.getMode());
        }

        if (this.getStaticCodeAnalysis() != null) {
            exerciseInfo.setStaticCodeAnalysis(this.getStaticCodeAnalysis());
        }

        if (this.getAllowOfflineIDE() != null) {
            exerciseInfo.setAllowOfflineIDE(this.getAllowOfflineIDE());
        }

        if (this.getAllowOnlineEditor() != null) {
            exerciseInfo.setAllowOnlineEditor(this.getAllowOnlineEditor());
        }

        if (this.getShowTestNamesToStudents() != null) {
            exerciseInfo.setShowTestNamesToStudents(this.getShowTestNamesToStudents());
        }

        if (this.getSequentialTestRuns() != null) {
            exerciseInfo.setSequentialTestRuns(this.getSequentialTestRuns());
        }
        return exerciseInfo;
    }

    /**
     * Used to merge attributes of ExerciseInfo into an ArtemisExerciseInfo class
     * @param exerciseInfo to merge
     */
    public void appendExerciseInfo(ExerciseInfo exerciseInfo) {
        this.setMetadataVersion(exerciseInfo.getMetadataVersion());
        this.setType(exerciseInfo.getType());
        this.setTitle(exerciseInfo.getTitle());
        this.setLicense(exerciseInfo.getLicense());
        this.setKeyword(exerciseInfo.getKeyword());
        this.setFormat(exerciseInfo.getFormat());
        this.setStructure(exerciseInfo.getStructure());
        this.setProgrammingLanguage(exerciseInfo.getProgrammingLanguage());
        this.setLanguage(exerciseInfo.getLanguage());
        if (exerciseInfo.getCreator() != null) {
            this.setCreator(List.copyOf(exerciseInfo.getCreator()));
        }
        if (exerciseInfo.getContributor() != null) {
            this.setContributor(List.copyOf(exerciseInfo.getContributor()));
        }
        this.setDifficulty(exerciseInfo.getDifficulty());
        this.setSource(exerciseInfo.getSource());
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        ArtemisExerciseInfo that = (ArtemisExerciseInfo) o;
        return Objects.equals(getMaxPoints(), that.getMaxPoints()) && Objects.equals(getBonusPoints(), that.getBonusPoints()) && getMode() == that.getMode() && Objects.equals(getStaticCodeAnalysis(), that.getStaticCodeAnalysis()) && Objects.equals(getAllowOfflineIDE(), that.getAllowOfflineIDE()) && Objects.equals(getAllowOnlineEditor(), that.getAllowOnlineEditor()) && Objects.equals(getShowTestNamesToStudents(), that.getShowTestNamesToStudents()) && Objects.equals(getSequentialTestRuns(), that.getSequentialTestRuns());
    }

    @Override
    public int hashCode() {
        return Objects.hash(getMaxPoints(), getBonusPoints(), getMode(), getStaticCodeAnalysis(), getAllowOfflineIDE(), getAllowOnlineEditor(), getShowTestNamesToStudents(), getSequentialTestRuns());
    }

    public enum ExerciseMode {INDIVIDUAL, TEAM};
}
